/**
 * Based heavily on https://github.com/webpack/webpack/blob/master/hot/log-apply-result.js
 * Original copyright Tobias Koppers @sokra (MIT license)
 */

module.exports = function(updatedModules, renewedModules, context) {
	var { log } = context;
	var unacceptedModules = updatedModules.filter(function(moduleId) {
		return renewedModules && renewedModules.indexOf(moduleId) < 0;
	});

	if (unacceptedModules.length > 0) {
		log(
			'warning',
			"[HMR] The following modules couldn't be hot updated: (They would need a full reload!)"
		);
		unacceptedModules.forEach(function(moduleId) {
			log('warning', '[HMR]  - ' + moduleId);
		});
	}

	if (!renewedModules || renewedModules.length === 0) {
		log('info', '[HMR] Nothing hot updated.');
	} else {
		log('info', '[HMR] Updated modules:');
		renewedModules.forEach(function(moduleId) {
			if (typeof moduleId === 'string' && moduleId.indexOf('!') !== -1) {
				var parts = moduleId.split('!');
				log.groupCollapsed('info', '[HMR]  - ' + parts.pop());
				log('info', '[HMR]  - ' + moduleId);
				log.groupEnd('info');
			} else {
				log('info', '[HMR]  - ' + moduleId);
			}
		});
		var numberIds = renewedModules.every(function(moduleId) {
			return typeof moduleId === 'number';
		});
		if (numberIds)
			log(
				'info',
				'[HMR] Consider using the NamedModulesPlugin for module names.'
			);
	}
};
